==> simile-0.2.1.tar.gz: simile-0.2.1/simile/client.py <==
import httpx
from httpx import AsyncClient
from typing import List, Dict, Any, Optional, Union, Type
import uuid
from pydantic import BaseModel

from .models import (
    Population, Agent, DataItem,
    CreatePopulationPayload, CreateAgentPayload, CreateDataItemPayload, UpdateDataItemPayload,
    DeletionResponse, QualGenerationRequest, QualGenerationResponse, MCGenerationRequest, MCGenerationResponse
)
from .exceptions import (
    SimileAPIError, SimileAuthenticationError, SimileNotFoundError, SimileBadRequestError
)

DEFAULT_BASE_URL = "https://simile-api-3a83be7adae0.herokuapp.com/api/v1"
TIMEOUT_CONFIG = httpx.Timeout(5.0, read=30.0, write=30.0, pool=30.0)

class Simile:
    # Make exceptions available as class attributes for convenience
    APIError = SimileAPIError
    AuthenticationError = SimileAuthenticationError
    NotFoundError = SimileNotFoundError
    BadRequestError = SimileBadRequestError

    def __init__(self, api_key: str, base_url: str = DEFAULT_BASE_URL):
        if not api_key:
            raise ValueError("API key is required.")
        self.api_key = api_key
        self.base_url = base_url.rstrip('/')
        self._client = AsyncClient(
            headers={"X-API-Key": self.api_key},
            timeout=TIMEOUT_CONFIG
        )

    async def _request(self, method: str, endpoint: str, **kwargs) -> Union[httpx.Response, BaseModel]:
        url = f"{self.base_url}/{endpoint.lstrip('/')}"
        response_model_cls: Optional[Type[BaseModel]] = kwargs.pop("response_model", None)
        
        try:
            response = await self._client.request(method, url, **kwargs)
            response.raise_for_status()

            if response_model_cls:
                return response_model_cls(**response.json())
            else:
                return response
        except httpx.HTTPStatusError as e:
            status_code = e.response.status_code
            try:
                error_data = e.response.json()
                detail = error_data.get("detail", e.response.text)
            except Exception:
                detail = e.response.text

            if status_code == 401:
                raise SimileAuthenticationError(detail=detail)
            elif status_code == 404:
                raise SimileNotFoundError(detail=detail)
            elif status_code == 400:
                raise SimileBadRequestError(detail=detail)
            else:
                raise SimileAPIError(f"API request failed: {e}", status_code=status_code, detail=detail)
        except httpx.RequestError as e:
            raise SimileAPIError(f"Request error: {e}")

    # --- Population Endpoints ---
    async def create_population(self, name: str, description: Optional[str] = None) -> Population:
        """Creates a new population."""
        payload = CreatePopulationPayload(name=name, description=description)
        response_data = await self._request("POST", "populations/create", json=payload.model_dump(mode='json', exclude_none=True), response_model=Population)
        return response_data

    async def get_population(self, population_id: Union[str, uuid.UUID]) -> Population:
        response_data = await self._request("GET", f"populations/get/{str(population_id)}", response_model=Population)
        return response_data

    async def delete_population(self, population_id: Union[str, uuid.UUID]) -> DeletionResponse:
        response_data = await self._request("DELETE", f"populations/delete/{str(population_id)}", response_model=DeletionResponse)
        return response_data

    # --- Agent Endpoints ---
    async def create_agent(self, name: str, population_id: Optional[Union[str, uuid.UUID]] = None, agent_data: Optional[List[Dict[str, Any]]] = None) -> Agent:
        """Creates a new agent, optionally within a population and with initial data items."""
        # Ensure population_id is uuid if provided
        pop_id_uuid: Optional[uuid.UUID] = None
        if population_id:
            pop_id_uuid = uuid.UUID(str(population_id)) if not isinstance(population_id, uuid.UUID) else population_id
        
        payload = CreateAgentPayload(name=name, population_id=pop_id_uuid, agent_data=agent_data)
        response_data = await self._request("POST", "agents/create", json=payload.model_dump(mode='json', exclude_none=True), response_model=Agent)
        return response_data

    async def get_agent(self, agent_id: Union[str, uuid.UUID]) -> Agent:
        response_data = await self._request("GET", f"agents/get/{str(agent_id)}", response_model=Agent)
        return response_data

    async def delete_agent(self, agent_id: Union[str, uuid.UUID]) -> DeletionResponse:
        response_data = await self._request("DELETE", f"agents/delete/{str(agent_id)}", response_model=DeletionResponse)
        return response_data

    # --- Data Item Endpoints ---
    async def create_data_item(self, agent_id: Union[str, uuid.UUID], data_type: str, content: Any) -> DataItem:
        """Creates a new data item for a specific agent."""
        payload = CreateDataItemPayload(data_type=data_type, content=content)
        response_data = await self._request("POST", f"data_item/create/{str(agent_id)}", json=payload.model_dump(mode='json'), response_model=DataItem)
        return response_data

    async def get_data_item(self, data_item_id: Union[str, uuid.UUID]) -> DataItem:
        response_data = await self._request("GET", f"data_item/get/{str(data_item_id)}", response_model=DataItem)
        return response_data

    async def list_data_items(self, agent_id: Union[str, uuid.UUID], data_type: Optional[str] = None) -> List[DataItem]:
        params = {}
        if data_type:
            params["data_type"] = data_type
        # Ensure agent_id is string for URL
        agent_id_str = str(agent_id)
        raw_response = await self._request("GET", f"data_item/list/{agent_id_str}", params=params) 
        return [DataItem(**item) for item in raw_response.json()] 

    async def update_data_item(self, data_item_id: Union[str, uuid.UUID], content: Any) -> DataItem:
        """Updates an existing data item."""
        payload = UpdateDataItemPayload(content=content)
        response_data = await self._request("POST", f"data_item/update/{str(data_item_id)}", json=payload.model_dump(), response_model=DataItem)
        return response_data

    async def delete_data_item(self, data_item_id: Union[str, uuid.UUID]) -> DeletionResponse:
        response_data = await self._request("DELETE", f"data_item/delete/{str(data_item_id)}", response_model=DeletionResponse)
        return response_data

    # --- LLM Generation Methods ---
    async def generate_qual_response(self, agent_id: uuid.UUID, question: str) -> QualGenerationResponse:
        """Generates a qualitative response from an agent based on a question."""
        endpoint = f"/generation/qual/{str(agent_id)}"
        request_payload = QualGenerationRequest(question=question) # Create payload internally
        response_data = await self._request(
            "POST", 
            endpoint, 
            json=request_payload.model_dump(), 
            response_model=QualGenerationResponse
        )
        return response_data

    async def generate_mc_response(self, agent_id: uuid.UUID, question: str, options: List[str]) -> MCGenerationResponse:
        """Generates a multiple-choice response from an agent."""
        endpoint = f"generation/mc/{str(agent_id)}" # Ensured str(agent_id)
        request_payload = MCGenerationRequest(question=question, options=options) # Create payload internally
        response_data = await self._request("POST", endpoint, json=request_payload.model_dump(), response_model=MCGenerationResponse)
        return response_data

    async def aclose(self):
        await self._client.aclose() 

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.aclose()

==> simile-0.2.3.tar.gz: simile-0.2.3/simile/client.py <==
import httpx
from httpx import AsyncClient
from typing import List, Dict, Any, Optional, Union, Type
import uuid
from pydantic import BaseModel

from .models import (
    Population, Agent as AgentModel, DataItem, DeletionResponse,
    QualGenerationRequest, QualGenerationResponse, 
    MCGenerationRequest, MCGenerationResponse,
    CreatePopulationPayload, CreateAgentPayload, CreateDataItemPayload, UpdateDataItemPayload,
    InitialDataItemPayload,
    SurveySessionCreateResponse
)
from .resources import Agent, SurveySession
from .exceptions import (
    SimileAPIError, SimileAuthenticationError, SimileNotFoundError, SimileBadRequestError
)

DEFAULT_BASE_URL = "https://simile-api-3a83be7adae0.herokuapp.com/api/v1"
TIMEOUT_CONFIG = httpx.Timeout(5.0, read=30.0, write=30.0, pool=30.0)

class Simile:
    APIError = SimileAPIError
    AuthenticationError = SimileAuthenticationError
    NotFoundError = SimileNotFoundError
    BadRequestError = SimileBadRequestError

    def __init__(self, api_key: str, base_url: str = DEFAULT_BASE_URL):
        if not api_key:
            raise ValueError("API key is required.")
        self.api_key = api_key
        self.base_url = base_url.rstrip('/')
        self._client = AsyncClient(
            headers={"X-API-Key": self.api_key},
            timeout=TIMEOUT_CONFIG
        )

    async def _request(self, method: str, endpoint: str, **kwargs) -> Union[httpx.Response, BaseModel]:
        url = f"{self.base_url}/{endpoint.lstrip('/')}"
        response_model_cls: Optional[Type[BaseModel]] = kwargs.pop("response_model", None)
        
        try:
            response = await self._client.request(method, url, **kwargs)
            response.raise_for_status()

            if response_model_cls:
                return response_model_cls(**response.json())
            else:
                return response
        except httpx.HTTPStatusError as e:
            status_code = e.response.status_code
            try:
                error_data = e.response.json()
                detail = error_data.get("detail", e.response.text)
            except Exception:
                detail = e.response.text

            if status_code == 401:
                raise SimileAuthenticationError(detail=detail)
            elif status_code == 404:
                raise SimileNotFoundError(detail=detail)
            elif status_code == 400:
                raise SimileBadRequestError(detail=detail)
            else:
                raise SimileAPIError(f"API request failed: {e}", status_code=status_code, detail=detail)
        except httpx.RequestError as e:
            raise SimileAPIError(f"Request error: {e}")

    def agent(self, agent_id: uuid.UUID) -> Agent:
        """Returns an Agent object to interact with a specific agent."""
        return Agent(agent_id=agent_id, client=self)

    async def create_survey_session(self, agent_id: uuid.UUID) -> SurveySession:
        """Creates a new survey session for the given agent and returns a SurveySession object."""
        endpoint = "sessions/"
        response_data = await self._request(
            "POST",
            endpoint,
            json={"agent_id": str(agent_id)},
            response_model=SurveySessionCreateResponse
        )
        return SurveySession(
            id=response_data.id,  
            agent_id=response_data.agent_id, 
            status=response_data.status,
            client=self
        )

    async def create_population(self, name: str, description: Optional[str] = None) -> Population:
        """Creates a new population."""
        payload = CreatePopulationPayload(name=name, description=description)
        response_data = await self._request("POST", "populations/create", json=payload.model_dump(mode='json', exclude_none=True), response_model=Population)
        return response_data

    async def get_population(self, population_id: Union[str, uuid.UUID]) -> Population:
        response_data = await self._request("GET", f"populations/get/{str(population_id)}", response_model=Population)
        return response_data

    async def delete_population(self, population_id: Union[str, uuid.UUID]) -> DeletionResponse:
        response_data = await self._request("DELETE", f"populations/delete/{str(population_id)}", response_model=DeletionResponse)
        return response_data

    async def get_agents_in_population(self, population_id: Union[str, uuid.UUID]) -> List[AgentModel]:
        """Retrieves all agents belonging to a specific population."""
        endpoint = f"populations/{str(population_id)}/agents"
        raw_response = await self._request("GET", endpoint)
        agents_data_list = raw_response.model_dump_json()
        return [AgentModel(**data) for data in agents_data_list]

    async def create_agent(self, name: str, population_id: Optional[Union[str, uuid.UUID]] = None, agent_data: Optional[List[Dict[str, Any]]] = None) -> AgentModel:
        """Creates a new agent, optionally within a population and with initial data items."""
        pop_id_uuid: Optional[uuid.UUID] = None
        if population_id:
            pop_id_uuid = uuid.UUID(str(population_id)) if not isinstance(population_id, uuid.UUID) else population_id
        
        payload = CreateAgentPayload(name=name, population_id=pop_id_uuid, agent_data=agent_data)
        response_data = await self._request("POST", "agents/create", json=payload.model_dump(mode='json', exclude_none=True), response_model=AgentModel)
        return response_data

    async def get_agent(self, agent_id: Union[str, uuid.UUID]) -> AgentModel:
        response_data = await self._request("GET", f"agents/get/{str(agent_id)}", response_model=AgentModel)
        return response_data

    async def delete_agent(self, agent_id: Union[str, uuid.UUID]) -> DeletionResponse:
        response_data = await self._request("DELETE", f"agents/delete/{str(agent_id)}", response_model=DeletionResponse)
        return response_data

    async def create_data_item(self, agent_id: Union[str, uuid.UUID], data_type: str, content: Any) -> DataItem:
        """Creates a new data item for a specific agent."""
        payload = CreateDataItemPayload(data_type=data_type, content=content)
        response_data = await self._request("POST", f"data_item/create/{str(agent_id)}", json=payload.model_dump(mode='json'), response_model=DataItem)
        return response_data

    async def get_data_item(self, data_item_id: Union[str, uuid.UUID]) -> DataItem:
        response_data = await self._request("GET", f"data_item/get/{str(data_item_id)}", response_model=DataItem)
        return response_data

    async def list_data_items(self, agent_id: Union[str, uuid.UUID], data_type: Optional[str] = None) -> List[DataItem]:
        params = {}
        if data_type:
            params["data_type"] = data_type
        agent_id_str = str(agent_id)
        raw_response = await self._request("GET", f"data_item/list/{agent_id_str}", params=params) 
        return [DataItem(**item) for item in raw_response.model_dump_json()] 

    async def update_data_item(self, data_item_id: Union[str, uuid.UUID], content: Any) -> DataItem:
        """Updates an existing data item."""
        payload = UpdateDataItemPayload(content=content)
        response_data = await self._request("POST", f"data_item/update/{str(data_item_id)}", json=payload.model_dump(), response_model=DataItem)
        return response_data

    async def delete_data_item(self, data_item_id: Union[str, uuid.UUID]) -> DeletionResponse:
        response_data = await self._request("DELETE", f"data_item/delete/{str(data_item_id)}", response_model=DeletionResponse)
        return response_data

    async def generate_qual_response(self, agent_id: uuid.UUID, question: str) -> QualGenerationResponse:
        """Generates a qualitative response from an agent based on a question."""
        endpoint = f"/generation/qual/{str(agent_id)}"
        request_payload = QualGenerationRequest(question=question)
        response_data = await self._request(
            "POST", 
            endpoint, 
            json=request_payload.model_dump(), 
            response_model=QualGenerationResponse
        )
        return response_data

    async def generate_mc_response(self, agent_id: uuid.UUID, question: str, options: List[str]) -> MCGenerationResponse:
        """Generates a multiple-choice response from an agent."""
        endpoint = f"generation/mc/{str(agent_id)}"
        request_payload = MCGenerationRequest(question=question, options=options)
        response_data = await self._request(
            "POST", 
            endpoint, 
            json=request_payload.model_dump(), 
            response_model=MCGenerationResponse
        )
        return response_data

    async def aclose(self):
        await self._client.aclose() 

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.aclose()
